#!/bin/sh
# \
exec wish "$0" "$@"
package require BLT
package require Tktable 2.5

option add *Font {Helvetica -12 bold} 
option add *Entry.Background snow 
option add *Entry.Font {Courier -18}
option add *Label.Font {Helvetica -14 bold}

set auto_path [linsert $auto_path 0 .]
query_init
create_images 

frame .top
pack [frame .top.fr -relief sunken -bd 2] -padx 5 -pady 5 
pack [button .top.fr.quit -text quit -command exit] \
		-side left -padx 10 -pady 5 
pack .top  -side bottom -fill x

frame .left
::blt::hierbox .left.hb -selectmode single -hideroot 1 \
	-yscrollcommand {.left.sb set} 
scrollbar .left.sb -width 10 -command {.left.hb yview} 
pack .left.sb -side right -fill y 
pack .left.hb -side left -fill both 

frame .right -width 600 -height 350 -relief sunken -bd 2
pack .left -side left -fill both -padx 10 -pady 10
pack .right -side left -expand 1 -fill both -padx 10 -pady 10
pack propagate .right 0

foreach i [DB_databases] {
	.left.hb insert end $i -icons img_db
	foreach r [DB_classes $i] {
		.left.hb insert -at $i end $r -icons img_table
	}
}

menu .left.hb.m -tearoff 0
.left.hb.m add command -label "new database"
.left.hb.m add command -label "new table"
.left.hb.m add command -label "edit table" -command db_edit_table
.left.hb.m add command -label "delete table"
.left.hb.m add command -label "delete database"

bind .left.hb <3> {
	tk_popup .left.hb.m [winfo pointerx .] [winfo pointery .] 
}

bind .left.hb <1> { db_set_table }

proc db_set_table {} {
	global tblval tblrow 
	catch {unset tblrow}
	set tblrow -1
	set tblval 0
	set ls [join [.left.hb get -full [.left.hb curselection]]]
	set db [lindex $ls 1]
	set tbl [lindex $ls 2]
	global $tbl
	if {[winfo exists .t]} {
		destroy .t .vs .hs
	}
	if {$tbl != ""} {
		table .t -borderwidth 2 -selectmode browse -autoclear 0 \
			-titlecols 1 -titlerows 1 -height 7 -width 5\
			-yscrollcommand [list .vs set] -xscrollcommand [list .hs set] \
			-rowheight -20 -command "db_cellcmd $tbl %r %c %i %s"  
		scrollbar .vs -width 10 -orient vertical -command [list .t yview]
		scrollbar .hs -width 10 -orient horizontal -command [list .t xview]
		pack .hs -in .right -side bottom -fill x
		pack .vs -in .right -side right -fill y
		pack .t -in .right -side right -expand 1 -fill both
		.t tag config title -bd 2 -fg black -bg #c0c0a8 -relief raised \
			-state normal
		.t tag config all -bg white -bd 1 -relief ridge -anchor w 
		.t tag config sel -bg white -bd 1 -relief ridge
		.t tag config active -bg #e0c8c8 -bd 1 -relief sunken
		query_opendb $db
		catch {$tbl clear}
		query $tbl -multiple 1
		$tbl select

		set cols [expr max([llength [$tbl attributes]]+1,2)]
		set rows [expr max([$tbl maxtuple]+2,2)]
		.t configure -cols $cols -rows $rows	
		.t width 0 2
		set j 1
		foreach attr [$tbl attributes] {
			.t width $j [lindex $attr 1]
			.t tag col all $j
			incr j
		}
		.t configure -browsecommand "db_update_tuple $tbl %r %c"
		bind .t <Shift-Delete> "db_delete_tuple $tbl" 
		bind .t <Return> "db_cell_advance ; break"
	}
}

#------------------------------------------------------------------
# table data editing
#------------------------------------------------------------------

proc db_cell_advance {} {
	set r 0
	catch {set r [.t index active row]}
	set c [.t index active col]
	set cmax [.t cget -cols]
	if {$c == [expr $cmax-1]} {
		incr r 
		set c 1
	} else {
		incr c
	}
	.t activate $r,$c
	.t see active
}

proc db_delete_tuple { tbl } {
	global tblrow $tbl
	set r [.t index active row]
	set tblrow -1
	.t activate 0,0
	$tbl set [expr $r-1]
	$tbl delete
	.t delete rows $r
	.t activate $r,1
	$tbl select
	.t config -command [.t cget -command]
}

proc db_cellcmd { tbl r c i s } {
	global tblrow tblval $tbl
	upvar #0 $tbl tb
	if {$r==0} {
		set at [lindex [$tbl attributes] [expr $c-1]]
		return [lindex $at 0]
	} 
	if {$c>0} {
		if {$i==0} {
			set at [lindex [$tbl attributes] [expr $c-1]]
			set v ${tbl}([expr $r-1],[lindex $at 0])
			set result ""
			catch { set result [set $v] }
			return [string trim $result]
		} else {
			if {($r>0) && ($c>0)} {
				set at [lindex [$tbl attributes] [expr $c-1]]
				#puts "set tb([expr $r-1],[lindex $at 0]) $s"
				set tb([expr $r-1],[lindex $at 0]) $s
				set tblval 1
			}
		}
	}
}

proc db_update_tuple { tbl r c } {
	global tblrow tblval $tbl 
	if {$tblrow != $r} {
		if {$tblval == 0} {
			set tblrow $r
			return
		}
		set tblval 0
		if {$tblrow > 0} {
			if {$tblrow > [$tbl maxtuple]} {
				set tblrow -1
				set act [.t index active]
				.t activate 0,0
				$tbl set new
				$tbl insert
				$tbl select
				.t activate $act
				.t config -command [.t cget -command] 
			} else {
				$tbl set [expr $tblrow-1]
				$tbl update
				$tbl select
			}
		}
		set tblrow $r
	} else {
		set rmax [.t cget -rows]
		if {$tblval == 0} { 
			return 
		}
		if {$r == [expr $rmax-1]} {
			.t insert rows $rmax
		}
	}
}

#------------------------------------------------------------------
# table attributes editing
#------------------------------------------------------------------

proc db_select_table { } {
	set ls [join [.left.hb get -full [.left.hb curselection]]]
	if {[llength $ls] <= 2} {
		error "no table selected.\nPlease select a table first"
	}
	#puts "editing table [lindex $ls 2]"
	return $ls
}

proc db_edittbl_adjust { adj } {
	global edittbl
	switch -- $adj {
	+1  { 
			set i $edittbl(i)
			set edittbl($i,attname) $edittbl(field) 
			set edittbl($i,attlen) $edittbl(fieldwidth)
			set edittbl($i,atttyp) $edittbl(type)
			set edittbl($i,del) $edittbl(del)
			if {[string compare new_field $edittbl($i,attname)]!=0} {
				incr edittbl(i)
				set i $edittbl(i)
				if {![info exists edittbl($i,attname)]} {
					set edittbl($i,attname) new_field 
					set edittbl($i,attlen) 20
					set edittbl($i,atttyp) char
					set edittbl($i,del) 0
				}
			}
		}
	-1  { 
			set i $edittbl(i)
			set edittbl($i,attname) $edittbl(field) 
			set edittbl($i,attlen) $edittbl(fieldwidth)
			set edittbl($i,atttyp) $edittbl(type)
			set edittbl($i,del) $edittbl(del)
			if  {$edittbl(i) > 0} { 
				incr edittbl(i) -1 
			}
		}
	clear {
			foreach n [array names edittbl(*,*)] {
				unset edittbl($n)
			}
			set edittbl(0,attname) new_field
			set edittbl(0,attlen) 20
			set edittbl(0,atttyp) char
			set edittbl(0,del) 0
			set edittbl(i) 0
		}
	store {
			set i $edittbl(i)
			set edittbl($i,attname) $edittbl(field) 
			set edittbl($i,attlen) $edittbl(fieldwidth)
			set edittbl($i,atttyp) $edittbl(type)
			set edittbl($i,del) $edittbl(del)
		}
	default { set edittbl(i) $adj }
	}
	set i $edittbl(i)
	set edittbl(field) $edittbl($i,attname)
	set edittbl(fieldwidth) $edittbl($i,attlen)
	set edittbl(type) $edittbl($i,atttyp)
	if {[info exists edittbl($i,del)]} {
		set edittbl(del) $edittbl($i,del)
	}
}

proc db_edit_table { } {
	global edittbl
	catch {unset edittbl}
	set ls [db_select_table]
	set db [lindex $ls 1]
	set tbl [lindex $ls 2]
	set f .edittbl
	set edittbl(type) char
	set edittbl(ok) 0

	DB_attrprop $db $tbl edittbl 
	set edittbl(del) 0
	db_edittbl_adjust 0

	if {[Dialog_Create $f "Edit Table" -borderwidth 10]} {
		label $f.tit -text "Table $tbl" -font {Helvetica -24 bold}
		label $f.lb1 -text "field name"
		entry $f.e -textvariable edittbl(field) -width 40
		label $f.lb2 -text "field width"
		entry $f.width -textvariable edittbl(fieldwidth) -width 3
		set r [frame $f.r -bd 2 -relief sunken]
		radiobutton $r.rb1 -text smallinteger \
			-variable edittbl(type) -value int2 
		radiobutton $r.rb2 -text integer  \
			-variable edittbl(type) -value int4 
		radiobutton $r.rb3 -text float  \
			-variable edittbl(type) -value float
		radiobutton $r.rb4 -text money  \
			-variable edittbl(type) -value money
		radiobutton $r.rb5 -text date \
			-variable edittbl(type) -value date 
		radiobutton $r.rb6 -text char  \
			-variable edittbl(type) -value char 
		radiobutton $r.rb7 -text varchar  \
			-variable edittbl(type) -value varchar 
		pack $r.rb1 $r.rb2 $r.rb3 $r.rb4 $r.rb5 $r.rb6 $r.rb7 -anchor nw
		label $f.lb3 -text "field#"
		label $f.lb4 -textvariable edittbl(i)
		checkbutton $f.del -text delete -variable edittbl(del)
		grid $f.tit -row 0 -column 0 -columnspan 3
		grid $f.del -row 1 -column 2 
		grid $f.lb1 -row 1 -column 0 -sticky sw
		grid $f.e  -row 2 -column 0 -columnspan 3 -sticky nw
		grid $f.lb2 -row 3 -column 0 -sticky sw
		grid $f.width -row 4 -column 0 -sticky nw
		grid $f.lb3 -row 3 -column 1 -sticky e
		grid $f.lb4 -row 3 -column 2 -sticky w
		grid $f.r  -row 0 -column 3 -rowspan 7 -padx 10
		set b [frame $f.buttons]
		grid $b -row 5 -column 0 -columnspan 2 -rowspan 3 
		button $b.next -text "next >>" -command { db_edittbl_adjust +1 }
		button $b.ok -text finish -command {set edittbl(ok) 1}
		button $b.cancel -text cancel -command {set edittbl(ok) 0}
		button $b.prev -text "<< prev" -command { db_edittbl_adjust -1 }
		pack $b.ok $b.cancel $b.prev $b.next \
			-side left -expand 1 -pady 5
	}
	$f.tit config -text "Table $tbl"
	Dialog_Wait $f edittbl(ok) $f.e
	Dialog_Dismiss $f
# update table definition, when finished
	if {$edittbl(ok)} {
		db_edittbl_adjust store 
		DB_attrprop $db $tbl origtbl 
		set rest {}
		set common {}
		foreach attname [array names edittbl *,attname] {
			set n [lindex [split $attname ,] 0]
			lappend all $n
			if {$edittbl($n,del) == 1} {
				continue
			}
			if {	[info exists origtbl($attname)] && \
					($origtbl($n,attname) == $edittbl($n,attname))  && \
					($origtbl($n,attlen)  == $edittbl($n,attlen))  && \
					($origtbl($n,atttyp)  == $edittbl($n,atttyp))  } {
					lappend common $n	
			} elseif {$edittbl($n,attname) != "new_field"} {
					lappend rest $n
			}
		}
# copy comman attributes
		set conn [pg_connect $db]		
		set qry "select " 
		set vg 0
		foreach n $common {
			if {$vg} { 
				append qry , 
			}
			incr vg
			append qry "$origtbl($n,attname)" 
		}
		append qry " into table tmp_edittbl from $tbl "
#puts "COPY: $qry"
		set handle [pg_exec $conn $qry]
		pg_result $handle -clear
# insert new attributes
		foreach n $rest {
			set qry "alter table tmp_edittbl add ( $edittbl($n,attname) "
			if {$edittbl($n,atttyp) == "char"} {
				append qry "$edittbl($n,atttyp)\($edittbl($n,attlen)\) )"
			} else {
				append qry "$edittbl($n,atttyp) )"
			}
#puts "APPEND: $qry"
			set handle [pg_exec $conn $qry]
			pg_result $handle -clear
		}
		set qry "drop table $tbl"
		set handle [pg_exec $conn $qry]
		pg_result $handle -clear
		set qry "alter table tmp_edittbl rename to $tbl"
		set handle [pg_exec $conn $qry]
		pg_result $handle -clear
		pg_disconnect $conn
	}
}

